/*
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "src/include/processor.h"
#include "src/include/types.h"

#include <iostream>

using namespace ppds;

int main() {
    std::cout << "### \033[1mBASIC_TEST\033[0m ###" << std::endl;
    std::cout << "Execution: ";

    PlanNode scan(NodeType::TableScan, {{"file_path", "data/basic_test/table_a.tbl"}}, {});
    PlanNode pred(NodeType::Predicate, {{"column_id", "0"}, {"condition", ">"}, {"literal", "1"}}, {scan});
    PlanNode scan2(NodeType::TableScan, {{"file_path", "data/basic_test/table_b.tbl"}}, {});
    PlanNode proj(NodeType::Projection, {{"column_ids", "0"}}, {scan2});
    PlanNode join(NodeType::EquiJoin, {{"left_column_id", "1"}, {"right_column_id", "0"}}, {pred, proj});
    PlanNode groupby(NodeType::GroupBy, {{"group_by_column_id", "2"}, {"aggregate_column_id", "0"}, {"aggregate_type", "max"}}, {join});

    std::vector<Record> results;
    try {
        results = Processor::ExecuteQuery(groupby);
    } catch (const std::exception& e) {
        std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
        std::cout << e.what() << std::endl;
        return 1;
    }

    std::cout << "\033[1;32mPASS\033[0m" << std::endl;

    std::cout << "Result evaluation: ";

    if (results.size() != 3) {
        std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
        std::cout << "Expected: 3 rows. Found: " << results.size();
        return 1;
    }
    if (results[0].GetColumnCount() != results[1].GetColumnCount()) {
        std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
        std::cout << "Inconclusive column count.";
        return 1;
    }
    if (results[1].GetColumnCount() != results[2].GetColumnCount()) {
        std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
        std::cout << "Inconclusive column count.";
        return 1;
    }
    if (results[0].GetColumnCount() != 2) {
        std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
        std::cout << "Expected: 2 columns. Found: " << results[0].GetColumnCount();
        return 1;
    }
    if (results[0].GetColumnType(0) != DataType::String) {
        std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
        std::cout << "Expected type for column 0: String";
        return 1;
    }
    if (results[0].GetColumnType(1) != DataType::Integer) {
        std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
        std::cout << "Expected type for column 0: Integer";
        return 1;
    }

    std::vector<std::string> result_strings{"def", "abc", "aab"};
    std::vector<int32_t> result_ints{4, 9, 5};
    for (uint16_t i = 0; i < 3; i++) {
        bool found_tuple = false;
        for (auto& record : results) {
            auto& value_1 = std::get<std::string>(record.GetValue(0));
            auto& value_2 = std::get<int32_t>(record.GetValue(1));
            if (value_1 == result_strings[i] && value_2 == result_ints[i]) {
                found_tuple = true;
                break;
            }
        }

        if (!found_tuple) {
            std::cout << "\033[1;31mFAIL\033[0m" << std::endl;
            std::cout << "Expected:\naab|5|\nabc|9|\ndef|4|" << std::endl;
            std::cout << "Found:\n";
            for (auto& record : results) {
                std::cout << std::get<std::string>(record.GetValue(0)) << "|" <<
                          std::get<int32_t>(record.GetValue(1)) << "|\n";
            }
            return 1;
        }
    }

    std::cout << "\033[1;32mPASS\033[0m" << std::endl;

    return 0;
}
